import bpy
import os
from bpy.props import StringProperty, BoolProperty
from bpy.types import Operator
from bpy_extras.io_utils import ImportHelper

# 2023-03-17
bl_info = {
   "name": "Setup VDM brush",
   "author": "Jun Mizutani",
   "version": (0, 2),
   "blender": (3, 5, 0),
   "location": "3D View",
   "description": "Change brush settings between Draw Brush and VDM brush",
   "warning": "",
   "category": "Sculpt"
}

# update checkbox
def vdm_update_callback(self, context):
    scene = context.scene
    brush = bpy.data.brushes["SculptDraw"]
    if (scene.prop_vdm == True):
        brush.stroke_method = 'ANCHORED'
        brush.curve_preset = 'CONSTANT'
        brush.texture_slot.map_mode = 'AREA_PLANE'
        brush.use_color_as_displacement = True
    else:
        brush.stroke_method = 'SPACE'
        brush.curve_preset = 'SMOOTH'
        brush.texture_slot.map_mode = 'TILED'
        brush.use_color_as_displacement = False
        brush.texture_slot.texture = None

# VDM Panel
class VDM_PT_Panel(bpy.types.Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Tool'
    bl_label = 'VDM'

    # draw checkbox only in sculpt mode
    @classmethod
    def poll(cls, context):
        if context.mode == "SCULPT":
            if context.tool_settings.sculpt.brush.name == 'SculptDraw':
                return True
        return False

    def draw(self, context):
        scene = context.scene
        layout = self.layout
        layout.prop(scene, "prop_vdm", text="VDM Brush")
        layout.operator(VDM_OT_Button.bl_idname,text="Load VDM Image")
        layout.operator(VDM_OT_MakeButton.bl_idname,text="VDM Shader")

# [Load VDM Brush] Button
class VDM_OT_Button(bpy.types.Operator):
    bl_idname = "vdm.load"
    bl_label = "Load VDM"

    def execute(self, context):
        bpy.ops.vdm.open_filebrowser('INVOKE_DEFAULT')
        return{'FINISHED'}

# [VDM Shader] Button
class VDM_OT_MakeButton(bpy.types.Operator):
    bl_idname = "vdm.material"
    bl_label = "VDM Shader"

    def execute(self, context):
        # material
        material = bpy.data.materials.new(name="VDM_material")
        material.use_nodes = True
        node_tree = material.node_tree
        node_tree.nodes.remove(node_tree.nodes.get('Principled BSDF'))
        material_output = node_tree.nodes.get('Material Output')
        # add nodes
        tex_coord = node_tree.nodes.new('ShaderNodeTexCoord')
        multadd = node_tree.nodes.new('ShaderNodeVectorMath')
        subtract = node_tree.nodes.new('ShaderNodeVectorMath')
        teximage = node_tree.nodes.new('ShaderNodeTexImage')
        multadd.inputs[1].default_value = (2.0, 2.0, 0)
        multadd.inputs[2].default_value = (-1.0, -1.0, 0)
        # operation
        subtract.operation = 'SUBTRACT'
        multadd.operation = 'MULTIPLY_ADD'
        # location
        tex_coord.location = (00, 200)
        multadd.location = (200, 200)
        subtract.location = (400, 200)
        material_output.location = (600, 200)
        teximage.location = (400, 50)
        # links
        links = node_tree.links
        links.new(tex_coord.outputs[2], multadd.inputs[0])
        links.new(tex_coord.outputs[3], subtract.inputs[0])
        links.new(multadd.outputs[0], subtract.inputs[1])
        links.new(subtract.outputs[0],material_output.inputs[0])
        # make teximage active
        node_tree.nodes.active = teximage
        return{'FINISHED'}

# Filebrowser
class OT_VDMOpenFilebrowser(Operator, ImportHelper):
     bl_idname = "vdm.open_filebrowser"
     bl_label = "Load VDM Image"

     filter_glob: StringProperty(
         default='*.exr',
         options={'HIDDEN'} )

     def execute(self, context):
         path = self.filepath
         dir, filename = os.path.split(path)
         new_texture = bpy.data.textures.new(filename, type='IMAGE')
         new_texture.image = bpy.data.images.load(filepath=path)
         return {'FINISHED'}

def init_props():
    scene = bpy.types.Scene
    scene.prop_vdm = BoolProperty(
        name="vdm",
        description="Enable VDM Brush",
        default=False, update=vdm_update_callback
    )

def clear_props():
    scene = bpy.types.Scene
    del scene.prop_vdm

def register():
    bpy.utils.register_class(VDM_OT_Button)
    bpy.utils.register_class(VDM_OT_MakeButton)
    bpy.utils.register_class(VDM_PT_Panel)
    bpy.utils.register_class(OT_VDMOpenFilebrowser)
    init_props()

def unregister():
    clear_props()
    bpy.utils.unregister_class(VDM_PT_Panel)
    bpy.utils.unregister_class(VDM_OT_Button)
    bpy.utils.unregister_class(VDM_OT_MakeButton)
    bpy.utils.unregister_class(OT_VDMOpenFilebrowser)

if __name__ == "__main__":
    register()
